// FILE: CSRTS6.CPP
// MODULE: DATE_CONVERT
// PROGRAM: CSRTS
////////////////////////////////////////////////////////////////////////////////
//
//     Functions:
//             date_convert()          converts dates in various formats to yyyymmdd format.
//
//             reverse_convert()       converts fields so that they sort in reverse.
//
//             number_convert()        processes number fields so that even negative
//                                     numbers will sort correctly.
//
////////////////////////////////////////////////////////////////////////////////

#include <fstream.h>
#include <string.h>
#include <stdio.h>
#include <math.h>
#include <ctype.h>
#include <stdlib.h>
#include <conio.h>
#include <dos.h>
#include <alloc.h>

#define max_string_len 134

char c_6_notice[] = " Copyright 1991, Ted Davis\
                   Sort routine after Comb Sort,\
                   Copyright 1991, McGraw-Hill";


void date_convert(char *string, int flag, char *year);
void reverse_convert(char *string);
void number_convert(char * string);


void date_convert(char *string, int flag, char *year)
// Converts dates from mm-dd-yyyy, mm-dd-yy, mm/dd/yyyy, mm/dd/yy, dd-mm-yyyy,
// dd-mm-yy, dd/mm/yyyy, and dd/mm/yy to yyyymmdd format.  Spaces are NOT acceptable.
// Missing years are replaced with the current one.  This routine does not check for
// AD/BC markers and corrupts dates between 1AD and 1000 AD; it is not suitable for
// converting wide ranging historical dates.  It does convert single digit day and
// month numbers to two digit form; the converted string has exactly eight characters
{
int i;
char * st;
char s[4];
char dd[4] = "";
char mm[4] = "";
char yy[6] = "";
    // replace all possible delimiters in the date string with spaces
     st = strchr(string, '-');          // find first -
     if(st != NULL)
     {
        i = st - string;                // index it
        string[i] = ' ';                // replace it with a space, making hours a separate field
        st = strchr(string, '-');       // find second
        if(st != NULL)                  // if there was one
        {
            i = st - string;            // index minutes/seconds separator, if any
            string[i] = ' ';            // replace it with a space, now three fields: hr, min, sec
        }
     }
     st = strchr(string, '/');          // find first /
     if(st != NULL)
     {
        i = st - string;                // index it
        string[i] = ' ';                // replace it with a space, making hours a separate field
        st = strchr(string, '/');       // find second
        if(st != NULL)                  // if there was one
        {
            i = st - string;            // index minutes/seconds separator, if any
            string[i] = ' ';            // replace it with a space, now three fields: hr, min, sec
        }
     }
    // parse into mm dd yy fields
    sscanf(string, "%s%s%s", mm, dd, yy);
    // build a new string in yyyymmdd order
    strcpy(string, yy);
    if(yy[0] == '\0')
    {           // fill missing year
        strcpy(string, year);
    }
    else
    {
        if(strlen(yy) == 2) // note: accepts three digit years, but resulting string is incorrect
        {      // fix missing century
            strcpy(string, year);
            string[2] = '\0';
            strcat(string, yy);
        }
    }
    // fix single digit day and month
    if(dd[1] == '\0')    // if the second character is the terminator, it is one digit
    {
        dd[2] = dd[1];  // shift the delimiter and character one place to the left
        dd[1] = dd[0];  // and prefix with a '0'
        dd[0] = '0';
    }
    if(mm[1] == '\0')    // if the second character is the terminator, it is one digit
    {
        mm[2] = mm[1];
        mm[1] = mm[0];
        mm[0] = '0';
    }
    if(flag)  // dd/mm sequence
    {
        strcat(string, dd);
        strcat(string, mm);
    }
    else      // mm/dd
    {
        strcat(string, mm);
        strcat(string, dd);
    }
}

void reverse_convert(char *string)
// Subtracts each character in the field from 255 so that
// the sort routine will see "larger" fields as "smaller", "smaller" as
// "larger", but "equal" still as "equal"
{
int i, j;
    j = strlen(string);
    for(i = 0; i < j; i++)
    {
        string[i] = (255 - string[i]);
    }
}

void number_convert(char * string, int i_width, int d_width)
// Field is in two parts: integer and fraction, which are processed separately, then
// joined with a decimal point to make one field.  Accepts numbers with $, %, #,
// etc. markers; accepts hex numbers.  Number may not contain whitespace.  Does
// not accept exponential notation.
// The decimal point, if present is changed to a space so that sscanf will see the
// number as two fields.  A complex test is performed for the special case of a
// decimal point that has no digits in front of it (which causes sscanf to parse
// the string incorrectly).  Extraneous characters are removed and the string is
// parsed into integer and fraction parts.  The integer is right justified in its
// specified field and the fraction is left justified in its.  Negative numbers are
// converted to sort in reverse order.  The first digit if the integer part of the
// number is made a space or a +, for negative or positive numbers.  Finally the
// integer and fraction are put back to together, separated by a decimal point.  The
// resulting string is zero filled and lined up so that the point is in the same position
// for each number processed with the same field widths.
{
int ni, nj, minus_flag = 0;
int fixup_decimal = 0;
char integer_part[max_string_len + 2] = "";
char fraction_part[max_string_len + 2] = "";
char * p;
    // check for a decimal point
    p = strchr(string, '.');
    if(p)
    {
        nj = p - string;
        string[nj] = ' ';
        if(!nj || (!isxdigit(string[nj-1])))
        // The object of this exercise is to flag the cases in which
        // the decimal point is not immediately preceded by a character in
        // the set 0123456789ABCDEFabcdef.
        {
            fixup_decimal = -1;
        }
    }
    nj = 0;
    // now strip out anything that is not a digit (hex accepted) or a space
    for(ni = 0; string[ni] != '\0'; ni++)
    {
        if(string[ni] == '-'){minus_flag = -1;} // flag negative numbers
        if((isxdigit(string[ni])) || (string[ni] == ' '))
        {
            string[nj++] = string[ni];
        }
    }
    string[nj] = '\0';
    sscanf(string, "%s%s", integer_part, fraction_part);
    // fix up for case where decimal point has no leading digit
    if(fixup_decimal)
    {
        strcpy(fraction_part, integer_part);
        strcpy(integer_part, "0");
    }
    // mark string as positive (tentatively)
    strcpy(string, "+");
    // pad out fraction_part with zeros
    for(ni = strlen(fraction_part); ni < d_width; ni++)
    {
        fraction_part[ni] = '0';
    }
    fraction_part[ni] = '\0'; // null terminate it
    // right justify integer
    nj = i_width + 1 - strlen(integer_part); // +1 for the sign marker
    for(ni = 1; ni < nj; ni++)
    {
        string[ni] = '0';   // leading zeros, leave sign marker as is
    }
    string[ni] = '\0';   //  null terminate string
    strcat(string, integer_part); // add the integer part
    if(minus_flag)
    {   //convert negative integer and fraction for reverse order sort
        for(nj = 1; string[nj] != '\0'; nj++)
        {
            string[nj] = 255 - string[nj];
        }
        for(nj = 0; fraction_part[nj] != '\0'; nj++)
        {
            fraction_part[nj] = 255 - fraction_part[nj];
        }
        // flag string as not positive
        string[0] = ' ';  // space tests "less than" '+'
    }
    strcat(string, ".");         //add a decimal point
    strcat(string, fraction_part); // add the fraction part
}

